@@ -35,7 +35,7 @@ const { spawn } = require('child_process');
 const { finished } = require('internal/streams/end-of-stream');
 const { resolve, sep, isAbsolute } = require('path');
 const { DefaultDeserializer, DefaultSerializer } = require('v8');
-const { getOptionValue } = require('internal/options');
+const { getOptionValue, getOptionsAsFlagsFromBinding } = require('internal/options');
 const { Interface } = require('internal/readline/interface');
 const { deserializeError } = require('internal/error_serdes');
 const { Buffer } = require('buffer');
@@ -78,6 +78,7 @@ const {
   kTestTimeoutFailure,
   Test,
 } = require('internal/test_runner/test');
+const { FastBuffer } = require('internal/buffer');
 
 const {
   convertStringToRegExp,
@@ -148,40 +149,61 @@ function getRunArgs(path, { forceExit,
                             only,
                             argv: suppliedArgs,
                             execArgv,
+                            rerunFailuresFilePath,
                             root: { timeout },
                             cwd }) {
-  const argv = ArrayPrototypeFilter(process.execArgv, filterExecArgv);
+  const processNodeOptions = getOptionsAsFlagsFromBinding();
+  const runArgs = ArrayPrototypeFilter(processNodeOptions, filterExecArgv);
+
+  /**
+   * Node supports V8 options passed via cli.
+   * These options are being consumed by V8 and are not stored in nodeOptions.
+   *
+   * We need to propagate these options to the child processes manually.
+   *
+   * An example of such option are --allow-natives-syntax and --expose-gc
+   */
+  const nodeOptionsSet = new SafeSet(processNodeOptions);
+  const unknownProcessExecArgv = ArrayPrototypeFilter(
+    process.execArgv,
+    (arg) => !nodeOptionsSet.has(arg),
+  );
+  ArrayPrototypePushApply(runArgs, unknownProcessExecArgv);
+
   if (forceExit === true) {
-    ArrayPrototypePush(argv, '--test-force-exit');
+    ArrayPrototypePush(runArgs, '--test-force-exit');
   }
   if (isUsingInspector()) {
-    ArrayPrototypePush(argv, `--inspect-port=${getInspectPort(inspectPort)}`);
+    ArrayPrototypePush(runArgs, `--inspect-port=${getInspectPort(inspectPort)}`);
   }
   if (testNamePatterns != null) {
-    ArrayPrototypeForEach(testNamePatterns, (pattern) => ArrayPrototypePush(argv, `--test-name-pattern=${pattern}`));
+    ArrayPrototypeForEach(testNamePatterns, (pattern) => ArrayPrototypePush(runArgs, `--test-name-pattern=${pattern}`));
   }
   if (testSkipPatterns != null) {
-    ArrayPrototypeForEach(testSkipPatterns, (pattern) => ArrayPrototypePush(argv, `--test-skip-pattern=${pattern}`));
+    ArrayPrototypeForEach(testSkipPatterns, (pattern) => ArrayPrototypePush(runArgs, `--test-skip-pattern=${pattern}`));
   }
   if (only === true) {
-    ArrayPrototypePush(argv, '--test-only');
+    ArrayPrototypePush(runArgs, '--test-only');
   }
   if (timeout != null) {
-    ArrayPrototypePush(argv, `--test-timeout=${timeout}`);
+    ArrayPrototypePush(runArgs, `--test-timeout=${timeout}`);
+  }
+  if (rerunFailuresFilePath) {
+    ArrayPrototypePush(runArgs, `--test-rerun-failures=${rerunFailuresFilePath}`);
   }
 
-  ArrayPrototypePushApply(argv, execArgv);
+  ArrayPrototypePushApply(runArgs, execArgv);
 
   if (path === kIsolatedProcessName) {
-    ArrayPrototypePush(argv, '--test');
-    ArrayPrototypePushApply(argv, ArrayPrototypeSlice(process.argv, 1));
+    ArrayPrototypePush(runArgs, '--test');
+    ArrayPrototypePushApply(runArgs, ArrayPrototypeSlice(process.argv, 1));
   } else {
-    ArrayPrototypePush(argv, path);
+    ArrayPrototypePush(runArgs, path);
   }
 
-  ArrayPrototypePushApply(argv, suppliedArgs);
+  ArrayPrototypePushApply(runArgs, suppliedArgs);
 
-  return argv;
+  return runArgs;
 }
 
 const serializer = new DefaultSerializer();
@@ -319,7 +341,7 @@ class FileTest extends Test {
     // This method is called when it is known that there is at least one message
     let bufferHead = this.#rawBuffer[0];
     let headerIndex = bufferHead.indexOf(v8Header);
-    let nonSerialized = Buffer.alloc(0);
+    let nonSerialized = new FastBuffer();
 
     while (bufferHead && headerIndex !== 0) {
       const nonSerializedData = headerIndex === -1 ?
@@ -381,7 +403,7 @@ function runTestFile(path, filesWatcher, opts) {
   const subtest = opts.root.createSubtest(FileTest, testPath, testOpts, async (t) => {
     const args = getRunArgs(path, opts);
     const stdio = ['pipe', 'pipe', 'pipe'];
-    const env = { __proto__: null, ...process.env, NODE_TEST_CONTEXT: 'child-v8' };
+    const env = { __proto__: null, NODE_TEST_CONTEXT: 'child-v8', ...(opts.env || process.env) };
     if (watchMode) {
       stdio.push('ipc');
       env.WATCH_REPORT_DEPENDENCIES = '1';
@@ -438,6 +460,9 @@ function runTestFile(path, filesWatcher, opts) {
       finished(child.stdout, { __proto__: null, signal: t.signal }),
     ]);
 
+    // Close readline interface to prevent memory leak
+    rl.close();
+
     if (watchMode) {
       filesWatcher.runningProcesses.delete(path);
       filesWatcher.runningSubtests.delete(path);
@@ -500,7 +525,7 @@ function watchFiles(testFiles, opts) {
   }
 
   // Watch for changes in current filtered files
-  watcher.on('changed', ({ owners, eventType }) => {
+  const onChanged = ({ owners, eventType }) => {
     if (!opts.hasFiles && (eventType === 'rename' || eventType === 'change')) {
       const updatedTestFiles = createTestFileList(opts.globPatterns, opts.cwd);
       const newFileName = ArrayPrototypeFind(updatedTestFiles, (x) => !ArrayPrototypeIncludes(testFiles, x));
@@ -541,19 +566,29 @@ function watchFiles(testFiles, opts) {
         triggerUncaughtException(error, true /* fromPromise */);
       }));
     }
-  });
+  };
+
+  watcher.on('changed', onChanged);
+
+  // Cleanup function to remove event listener and prevent memory leak
+  const cleanup = () => {
+    watcher.removeListener('changed', onChanged);
+    opts.root.harness.watching = false;
+    opts.root.postRun();
+  };
+
   if (opts.signal) {
     kResistStopPropagation ??= require('internal/event_target').kResistStopPropagation;
     opts.signal.addEventListener(
       'abort',
-      () => {
-        opts.root.harness.watching = false;
-        opts.root.postRun();
-      },
+      cleanup,
       { __proto__: null, once: true, [kResistStopPropagation]: true },
     );
   }
 
+  // Expose cleanup method for proper resource management
+  filesWatcher.cleanup = cleanup;
+
   return filesWatcher;
 }
 
@@ -587,6 +622,8 @@ function run(options = kEmptyObject) {
     execArgv = [],
     argv = [],
     cwd = process.cwd(),
+    rerunFailuresFilePath,
+    env,
   } = options;
 
   if (files != null) {
@@ -619,6 +656,10 @@ function run(options = kEmptyObject) {
     );
   }
 
+  if (rerunFailuresFilePath) {
+    validatePath(rerunFailuresFilePath, 'options.rerunFailuresFilePath');
+  }
+
   if (shard != null) {
     validateObject(shard, 'options.shard');
     // Avoid re-evaluating the shard object in case it's a getter
@@ -691,6 +732,14 @@ function run(options = kEmptyObject) {
     validatePath(globalSetupPath, 'options.globalSetupPath');
   }
 
+  if (env != null) {
+    validateObject(env);
+
+    if (isolation === 'none') {
+      throw new ERR_INVALID_ARG_VALUE('options.env', env, 'is not supported with isolation=\'none\'');
+    }
+  }
+
   const rootTestOptions = { __proto__: null, concurrency, timeout, signal };
   const globalOptions = {
     __proto__: null,
@@ -701,6 +750,7 @@ function run(options = kEmptyObject) {
     coverage,
     coverageExcludeGlobs,
     coverageIncludeGlobs,
+    rerunFailuresFilePath,
     lineCoverage: lineCoverage,
     branchCoverage: branchCoverage,
     functionCoverage: functionCoverage,
@@ -734,6 +784,8 @@ function run(options = kEmptyObject) {
     isolation,
     argv,
     execArgv,
+    rerunFailuresFilePath,
+    env,
   };
 
   if (isolation === 'process') {
@@ -822,7 +874,10 @@ function run(options = kEmptyObject) {
             );
             if (topLevelTestCount === root.subtests.length) {
               // This file had no tests in it. Add the placeholder test.
-              const subtest = root.createSubtest(Test, testFile);
+              const subtest = root.createSubtest(Test, testFile, kEmptyObject, undefined, {
+                __proto__: null,
+                loc: [1, 1, resolve(testFile)],
+              });
               if (threw) {
                 subtest.fail(importError);
               }
