@@ -22,6 +22,7 @@
 'use strict';
 
 const {
+  FunctionPrototypeCall,
   ObjectAssign,
   ObjectDefineProperty,
   ObjectSetPrototypeOf,
@@ -582,7 +583,7 @@ function TLSSocket(socket, opts) {
   // distinguishable from regular ones.
   this.encrypted = true;
 
-  ReflectApply(net.Socket, this, [{
+  FunctionPrototypeCall(net.Socket, this, {
     handle: this._wrapHandle(wrap, handle, wrapHasActiveWriteFromPrevOwner),
     allowHalfOpen: socket ? socket.allowHalfOpen : tlsOptions.allowHalfOpen,
     pauseOnCreate: tlsOptions.pauseOnConnect,
@@ -590,7 +591,7 @@ function TLSSocket(socket, opts) {
     highWaterMark: tlsOptions.highWaterMark,
     onread: !socket ? tlsOptions.onread : null,
     signal: tlsOptions.signal,
-  }]);
+  });
 
   // Proxy for API compatibility
   this.ssl = this._handle;  // C++ TLSWrap object
@@ -1390,7 +1391,7 @@ function Server(options, listener) {
   }
 
   // constructor call
-  ReflectApply(net.Server, this, [options, tlsConnectionListener]);
+  FunctionPrototypeCall(net.Server, this, options, tlsConnectionListener);
 
   if (listener) {
     this.on('secureConnection', listener);
@@ -1579,8 +1580,7 @@ Server.prototype[EE.captureRejectionSymbol] = function(
       sock.destroy(err);
       break;
     default:
-      ReflectApply(net.Server.prototype[SymbolFor('nodejs.rejection')], this,
-                   [err, event, sock]);
+      FunctionPrototypeCall(net.Server.prototype[SymbolFor('nodejs.rejection')], this, err, event, sock);
   }
 };
 
