@@ -7,6 +7,7 @@ const {
   ArrayPrototypeIncludes,
   ArrayPrototypeJoin,
   ArrayPrototypeMap,
+  NumberIsFinite,
   NumberIsInteger,
   NumberIsNaN,
   NumberMAX_SAFE_INTEGER,
@@ -23,6 +24,7 @@ const {
   codes: {
     ERR_INVALID_ARG_TYPE: { HideStackFramesError: ERR_INVALID_ARG_TYPE },
     ERR_INVALID_ARG_VALUE: { HideStackFramesError: ERR_INVALID_ARG_VALUE },
+    ERR_INVALID_THIS: { HideStackFramesError: ERR_INVALID_THIS },
     ERR_OUT_OF_RANGE: { HideStackFramesError: ERR_OUT_OF_RANGE },
     ERR_SOCKET_BAD_PORT: { HideStackFramesError: ERR_SOCKET_BAD_PORT },
     ERR_UNKNOWN_SIGNAL: { HideStackFramesError: ERR_UNKNOWN_SIGNAL },
@@ -33,6 +35,7 @@ const { normalizeEncoding } = require('internal/util');
 const {
   isAsyncFunction,
   isArrayBufferView,
+  isRegExp,
 } = require('internal/util/types');
 const { signals } = internalBinding('constants').os;
 
@@ -525,15 +528,21 @@ const validateLinkHeaderFormat = hideStackFrames((value, name) => {
   }
 });
 
-const validateInternalField = hideStackFrames((object, fieldKey, className) => {
+/**
+ * Validate provided `this` object by checking that it has specific own property
+ * @param {any} object
+ * @param {string|symbol} fieldKey
+ * @param {string} className
+ */
+const validateThisInternalField = hideStackFrames((object, fieldKey, className) => {
   if (typeof object !== 'object' || object === null || !ObjectPrototypeHasOwnProperty(object, fieldKey)) {
-    throw new ERR_INVALID_ARG_TYPE('this', className, object);
+    throw new ERR_INVALID_THIS(className);
   }
 });
 
 /**
  * @param {any} hints
- * @return {string}
+ * @returns {string}
  */
 const validateLinkHeaderValue = hideStackFrames((hints) => {
   if (typeof hints === 'string') {
@@ -567,6 +576,78 @@ const validateLinkHeaderValue = hideStackFrames((hints) => {
   );
 });
 
+/**
+ * Validates a single ignore option element (string, RegExp, or Function).
+ * @param {*} value
+ * @param {string} name
+ */
+const validateIgnoreOptionElement = hideStackFrames((value, name) => {
+  if (typeof value === 'string') {
+    if (value.length === 0)
+      throw new ERR_INVALID_ARG_VALUE(name, value, 'must be a non-empty string');
+    return;
+  }
+  if (isRegExp(value)) return;
+  if (typeof value === 'function') return;
+  throw new ERR_INVALID_ARG_TYPE(name, ['string', 'RegExp', 'Function'], value);
+});
+
+/**
+ * Validates the ignore option for fs.watch.
+ * @param {*} value
+ * @param {string} name
+ */
+const validateIgnoreOption = hideStackFrames((value, name) => {
+  if (value == null) return;
+  if (ArrayIsArray(value)) {
+    for (let i = 0; i < value.length; i++) {
+      validateIgnoreOptionElement(value[i], `${name}[${i}]`);
+    }
+    return;
+  }
+  validateIgnoreOptionElement(value, name);
+});
+
+// 1. Returns false for undefined and NaN
+// 2. Returns true for finite numbers
+// 3. Throws ERR_INVALID_ARG_TYPE for non-numbers
+// 4. Throws ERR_OUT_OF_RANGE for infinite numbers
+const validateFiniteNumber = hideStackFrames((number, name) => {
+  // Common case
+  if (number === undefined) {
+    return false;
+  }
+
+  if (NumberIsFinite(number)) {
+    return true; // Is a valid number
+  }
+
+  if (NumberIsNaN(number)) {
+    return false;
+  }
+
+  validateNumber(number, name);
+
+  // Infinite numbers
+  throw new ERR_OUT_OF_RANGE(name, 'a finite number', number);
+});
+
+// 1. Returns def for number when it's undefined or NaN
+// 2. Returns number for finite numbers >= lower and <= upper
+// 3. Throws ERR_INVALID_ARG_TYPE for non-numbers
+// 4. Throws ERR_OUT_OF_RANGE for infinite numbers or numbers > upper or < lower
+const checkRangesOrGetDefault = hideStackFrames(
+  (number, name, lower, upper, def) => {
+    if (!validateFiniteNumber(number, name)) {
+      return def;
+    }
+    if (number < lower || number > upper) {
+      throw new ERR_OUT_OF_RANGE(name, `>= ${lower} and <= ${upper}`, number);
+    }
+    return number;
+  },
+);
+
 module.exports = {
   isInt32,
   isUint32,
@@ -580,6 +661,7 @@ module.exports = {
   validateDictionary,
   validateEncoding,
   validateFunction,
+  validateIgnoreOption,
   validateInt32,
   validateInteger,
   validateNumber,
@@ -600,5 +682,7 @@ module.exports = {
   validateUnion,
   validateAbortSignal,
   validateLinkHeaderValue,
-  validateInternalField,
+  validateThisInternalField,
+  validateFiniteNumber,
+  checkRangesOrGetDefault,
 };
