@@ -36,6 +36,8 @@ const {
   ObjectDefineProperty,
   ObjectSetPrototypeOf,
   Symbol,
+  SymbolAsyncDispose,
+  SymbolDispose,
 } = primordials;
 
 const EventEmitter = require('events');
@@ -103,6 +105,7 @@ const {
     ERR_INVALID_FD_TYPE,
     ERR_INVALID_HANDLE_TYPE,
     ERR_INVALID_IP_ADDRESS,
+    ERR_IP_BLOCKED,
     ERR_MISSING_ARGS,
     ERR_SERVER_ALREADY_LISTEN,
     ERR_SERVER_NOT_RUNNING,
@@ -119,8 +122,6 @@ const {
   isWindows,
   kEmptyObject,
   promisify,
-  SymbolAsyncDispose,
-  SymbolDispose,
 } = require('internal/util');
 const {
   validateAbortSignal,
@@ -164,8 +165,15 @@ const {
 } = require('internal/perf/observe');
 const { getDefaultHighWaterMark } = require('internal/streams/state');
 
-function getFlags(ipv6Only) {
-  return ipv6Only === true ? TCPConstants.UV_TCP_IPV6ONLY : 0;
+function getFlags(options) {
+  let flags = 0;
+  if (options.ipv6Only === true) {
+    flags |= TCPConstants.UV_TCP_IPV6ONLY;
+  }
+  if (options.reusePort === true) {
+    flags |= TCPConstants.UV_TCP_REUSEPORT;
+  }
+  return flags;
 }
 
 function createHandle(fd, is_server) {
@@ -230,11 +238,6 @@ function connect(...args) {
   debug('createConnection', normalized);
   const socket = new Socket(options);
 
-  if (netClientSocketChannel.hasSubscribers) {
-    netClientSocketChannel.publish({
-      socket,
-    });
-  }
   if (options.timeout) {
     socket.setTimeout(options.timeout);
   }
@@ -373,7 +376,7 @@ function Socket(options) {
       'is not supported',
     );
   }
-  if (typeof options?.keepAliveInitialDelay !== 'undefined') {
+  if (options?.keepAliveInitialDelay !== undefined) {
     validateNumber(
       options?.keepAliveInitialDelay, 'options.keepAliveInitialDelay',
     );
@@ -503,6 +506,12 @@ function Socket(options) {
   // Used after `.destroy()`
   this[kBytesRead] = 0;
   this[kBytesWritten] = 0;
+  if (options.blockList) {
+    if (!module.exports.BlockList.isBlockList(options.blockList)) {
+      throw new ERR_INVALID_ARG_TYPE('options.blockList', 'net.BlockList', options.blockList);
+    }
+    this.blockList = options.blockList;
+  }
 }
 ObjectSetPrototypeOf(Socket.prototype, stream.Duplex.prototype);
 ObjectSetPrototypeOf(Socket, stream.Duplex);
@@ -747,8 +756,7 @@ Socket.prototype.resetAndDestroy = function() {
 };
 
 Socket.prototype.pause = function() {
-  if (this[kBuffer] && !this.connecting && this._handle &&
-      this._handle.reading) {
+  if (this[kBuffer] && !this.connecting && this._handle?.reading) {
     this._handle.reading = false;
     if (!this.destroyed) {
       const err = this._handle.readStop();
@@ -1046,10 +1054,10 @@ function internalConnect(
 
   if (localAddress || localPort) {
     if (addressType === 4) {
-      localAddress = localAddress || DEFAULT_IPV4_ADDR;
+      localAddress ||= DEFAULT_IPV4_ADDR;
       err = self._handle.bind(localAddress, localPort);
     } else { // addressType === 6
-      localAddress = localAddress || DEFAULT_IPV6_ADDR;
+      localAddress ||= DEFAULT_IPV6_ADDR;
       err = self._handle.bind6(localAddress, localPort, flags);
     }
     debug('connect: binding to localAddress: %s and localPort: %d (addressType: %d)',
@@ -1058,7 +1066,7 @@ function internalConnect(
     err = checkBindError(err, localPort, self._handle);
     if (err) {
       const ex = new ExceptionWithHostPort(err, 'bind', localAddress, localPort);
-      self.destroy(ex);
+      process.nextTick(emitErrorAndDestroy, self, ex);
       return;
     }
   }
@@ -1067,6 +1075,10 @@ function internalConnect(
   self.emit('connectionAttempt', address, port, addressType);
 
   if (addressType === 6 || addressType === 4) {
+    if (self.blockList?.check(address, `ipv${addressType}`)) {
+      process.nextTick(emitErrorAndDestroy, self, new ERR_IP_BLOCKED(address));
+      return;
+    }
     const req = new TCPConnectWrap();
     req.oncomplete = afterConnect;
     req.address = address;
@@ -1096,12 +1108,20 @@ function internalConnect(
     }
 
     const ex = new ExceptionWithHostPort(err, 'connect', address, port, details);
-    self.destroy(ex);
+    process.nextTick(emitErrorAndDestroy, self, ex);
   } else if ((addressType === 6 || addressType === 4) && hasObserver('net')) {
     startPerf(self, kPerfHooksNetConnectContext, { type: 'net', name: 'connect', detail: { host: address, port } });
   }
 }
 
+// Helper function to defer socket destruction to the next tick.
+// This ensures that error handlers have a chance to be set up
+// before the error is emitted, particularly important when using
+// http.request with a custom lookup function.
+function emitErrorAndDestroy(self, err) {
+  self.destroy(err);
+}
+
 
 function internalConnectMultiple(context, canceled) {
   clearTimeout(context[kTimeout]);
@@ -1115,11 +1135,11 @@ function internalConnectMultiple(context, canceled) {
   // All connections have been tried without success, destroy with error
   if (canceled || context.current === context.addresses.length) {
     if (context.errors.length === 0) {
-      self.destroy(new ERR_SOCKET_CONNECTION_TIMEOUT());
+      process.nextTick(emitErrorAndDestroy, self, new ERR_SOCKET_CONNECTION_TIMEOUT());
       return;
     }
 
-    self.destroy(new NodeAggregateError(context.errors));
+    process.nextTick(emitErrorAndDestroy, self, new NodeAggregateError(context.errors));
     return;
   }
 
@@ -1156,6 +1176,14 @@ function internalConnectMultiple(context, canceled) {
     }
   }
 
+  if (self.blockList?.check(address, `ipv${addressType}`)) {
+    const ex = new ERR_IP_BLOCKED(address);
+    ArrayPrototypePush(context.errors, ex);
+    self.emit('connectionAttemptFailed', address, port, addressType, ex);
+    internalConnectMultiple(context);
+    return;
+  }
+
   debug('connect/multiple: attempting to connect to %s:%d (addressType: %d)', address, port, addressType);
   self.emit('connectionAttempt', address, port, addressType);
 
@@ -1213,12 +1241,18 @@ Socket.prototype.connect = function(...args) {
   const options = normalized[0];
   const cb = normalized[1];
 
+  if (netClientSocketChannel.hasSubscribers) {
+    netClientSocketChannel.publish({
+      socket: this,
+    });
+  }
+
   if (cb !== null) {
     this.once('connect', cb);
   }
 
   // If the parent is already connecting, do not attempt to connect again
-  if (this._parent && this._parent.connecting) {
+  if (this._parent?.connecting) {
     return this;
   }
 
@@ -1286,6 +1320,8 @@ function lookupAndConnect(self, options) {
   const host = options.host || 'localhost';
   let { port, autoSelectFamilyAttemptTimeout, autoSelectFamily } = options;
 
+  validateString(host, 'options.host');
+
   if (localAddress && !isIP(localAddress)) {
     throw new ERR_INVALID_IP_ADDRESS(localAddress);
   }
@@ -1294,7 +1330,7 @@ function lookupAndConnect(self, options) {
     validateNumber(localPort, 'options.localPort');
   }
 
-  if (typeof port !== 'undefined') {
+  if (port !== undefined) {
     if (typeof port !== 'number' && typeof port !== 'string') {
       throw new ERR_INVALID_ARG_TYPE('options.port',
                                      ['number', 'string'], port);
@@ -1391,7 +1427,7 @@ function lookupAndConnect(self, options) {
         // calls net.Socket.connect() on it (that's us). There are no event
         // listeners registered yet so defer the error event to the next tick.
         process.nextTick(connectErrorNT, self, err);
-      } else if (!isIP(ip)) {
+      } else if ((typeof ip !== 'string') || !isIP(ip)) {
         err = new ERR_INVALID_IP_ADDRESS(ip);
         process.nextTick(connectErrorNT, self, err);
       } else if (addressType !== 4 && addressType !== 6) {
@@ -1446,9 +1482,7 @@ function lookupAndConnectMultiple(
           return;
         }
         if (isIP(ip) && (addressType === 4 || addressType === 6)) {
-          if (!destinations) {
-            destinations = addressType === 6 ? { 6: 0, 4: 1 } : { 4: 0, 6: 1 };
-          }
+          destinations ||= addressType === 6 ? { 6: 0, 4: 1 } : { 4: 0, 6: 1 };
 
           const destination = destinations[addressType];
 
@@ -1753,7 +1787,7 @@ function Server(options, connectionListener) {
   } else {
     throw new ERR_INVALID_ARG_TYPE('options', 'Object', options);
   }
-  if (typeof options.keepAliveInitialDelay !== 'undefined') {
+  if (options.keepAliveInitialDelay !== undefined) {
     validateNumber(
       options.keepAliveInitialDelay, 'options.keepAliveInitialDelay',
     );
@@ -1762,7 +1796,7 @@ function Server(options, connectionListener) {
       options.keepAliveInitialDelay = 0;
     }
   }
-  if (typeof options.highWaterMark !== 'undefined') {
+  if (options.highWaterMark !== undefined) {
     validateNumber(
       options.highWaterMark, 'options.highWaterMark',
     );
@@ -1787,6 +1821,12 @@ function Server(options, connectionListener) {
   this.keepAlive = Boolean(options.keepAlive);
   this.keepAliveInitialDelay = ~~(options.keepAliveInitialDelay / 1000);
   this.highWaterMark = options.highWaterMark ?? getDefaultHighWaterMark();
+  if (options.blockList) {
+    if (!module.exports.BlockList.isBlockList(options.blockList)) {
+      throw new ERR_INVALID_ARG_TYPE('options.blockList', 'net.BlockList', options.blockList);
+    }
+    this.blockList = options.blockList;
+  }
 }
 ObjectSetPrototypeOf(Server.prototype, EventEmitter.prototype);
 ObjectSetPrototypeOf(Server, EventEmitter);
@@ -1836,12 +1876,12 @@ function createServerHandle(address, port, addressType, fd, flags) {
       if (err) {
         handle.close();
         // Fallback to ipv4
-        return createServerHandle(DEFAULT_IPV4_ADDR, port);
+        return createServerHandle(DEFAULT_IPV4_ADDR, port, undefined, undefined, flags);
       }
     } else if (addressType === 6) {
       err = handle.bind6(address, port, flags);
     } else {
-      err = handle.bind(address, port);
+      err = handle.bind(address, port, flags);
     }
   }
 
@@ -2025,7 +2065,7 @@ Server.prototype.listen = function(...args) {
     toNumber(args.length > 2 && args[2]);  // (port, host, backlog)
 
   options = options._handle || options.handle || options;
-  const flags = getFlags(options.ipv6Only);
+  const flags = getFlags(options);
   //  Refresh the id to make the previous call invalid
   this._listeningId++;
   // (handle[, backlog][, cb]) where handle is an object with a handle
@@ -2047,7 +2087,7 @@ Server.prototype.listen = function(...args) {
   // or (options[, cb]) where options.port is explicitly set as undefined or
   // null, bind to an arbitrary unused port
   if (args.length === 0 || typeof args[0] === 'function' ||
-      (typeof options.port === 'undefined' && 'port' in options) ||
+      (options.port === undefined && 'port' in options) ||
       options.port === null) {
     options.port = 0;
   }
@@ -2058,6 +2098,9 @@ Server.prototype.listen = function(...args) {
   if (typeof options.port === 'number' || typeof options.port === 'string') {
     validatePort(options.port, 'options.port');
     backlog = options.backlog || backlogFromArgs;
+    if (options.reusePort === true) {
+      options.exclusive = true;
+    }
     // start TCP server listening on host:port
     if (options.host) {
       lookupAndListen(this, options.port | 0, options.host, backlog,
@@ -2065,7 +2108,7 @@ Server.prototype.listen = function(...args) {
     } else { // Undefined host, listens on unspecified address
       // Default addressType 4 will be used to search for primary server
       listenInCluster(this, null, options.port | 0, 4,
-                      backlog, undefined, options.exclusive);
+                      backlog, undefined, options.exclusive, flags);
     }
     return this;
   }
@@ -2184,7 +2227,7 @@ ObjectDefineProperty(Server.prototype, 'listening', {
 });
 
 Server.prototype.address = function() {
-  if (this._handle && this._handle.getsockname) {
+  if (this._handle?.getsockname) {
     const out = {};
     const err = this._handle.getsockname(out);
     if (err) {
@@ -2232,7 +2275,15 @@ function onconnection(err, clientHandle) {
     clientHandle.close();
     return;
   }
-
+  if (self.blockList && typeof clientHandle.getpeername === 'function') {
+    const remoteInfo = { __proto__: null };
+    clientHandle.getpeername(remoteInfo);
+    const addressType = isIP(remoteInfo.address);
+    if (addressType && self.blockList.check(remoteInfo.address, `ipv${addressType}`)) {
+      clientHandle.close();
+      return;
+    }
+  }
   const socket = new Socket({
     handle: clientHandle,
     allowHalfOpen: self.allowHalfOpen,
@@ -2352,7 +2403,7 @@ Server.prototype[SymbolAsyncDispose] = async function() {
   if (!this._handle) {
     return;
   }
-  return FunctionPrototypeCall(promisify(this.close), this);
+  await FunctionPrototypeCall(promisify(this.close), this);
 };
 
 Server.prototype._emitCloseIfDrained = function() {
@@ -2431,48 +2482,9 @@ Server.prototype.unref = function() {
   return this;
 };
 
-let _setSimultaneousAccepts;
-let warnSimultaneousAccepts = true;
-
-if (isWindows) {
-  let simultaneousAccepts;
-
-  _setSimultaneousAccepts = function(handle) {
-    if (warnSimultaneousAccepts) {
-      process.emitWarning(
-        'net._setSimultaneousAccepts() is deprecated and will be removed.',
-        'DeprecationWarning', 'DEP0121');
-      warnSimultaneousAccepts = false;
-    }
-    if (handle === undefined) {
-      return;
-    }
-
-    if (simultaneousAccepts === undefined) {
-      simultaneousAccepts = (process.env.NODE_MANY_ACCEPTS &&
-                             process.env.NODE_MANY_ACCEPTS !== '0');
-    }
-
-    if (handle._simultaneousAccepts !== simultaneousAccepts) {
-      handle.setSimultaneousAccepts(!!simultaneousAccepts);
-      handle._simultaneousAccepts = simultaneousAccepts;
-    }
-  };
-} else {
-  _setSimultaneousAccepts = function() {
-    if (warnSimultaneousAccepts) {
-      process.emitWarning(
-        'net._setSimultaneousAccepts() is deprecated and will be removed.',
-        'DeprecationWarning', 'DEP0121');
-      warnSimultaneousAccepts = false;
-    }
-  };
-}
-
 module.exports = {
   _createServerHandle: createServerHandle,
   _normalizeArgs: normalizeArgs,
-  _setSimultaneousAccepts,
   get BlockList() {
     BlockList ??= require('internal/blocklist').BlockList;
     return BlockList;
