@@ -9,8 +9,11 @@ const fs = require('fs');
 const assert = require('assert');
 const { join } = require('path');
 const { setTimeout } = require('timers/promises');
+const { skipIfNoWatch } = require('../common/watch');
 const tmpdir = require('../common/tmpdir');
 
+skipIfNoWatch();
+
 class WatchTestCase {
   constructor(dirName, files) {
     this.dirName = dirName;
